// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/samples/mbsync/main.c $
// $Rev: 53956 $
// $Date: 2023-11-29 06:55:21 -0600 (Wed, 29 Nov 2023) $

// 24DSI: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg				values	desc
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",			{ -1 },	"Continue testing until an error occurs."									},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",			{ +1 },	"Continue testing even if errors occur."									},
		{	GSC_ARG_S32_FLAG,	&args->star,			"-d",			{ 0 },	"The clock and sync signals are in a DAISY CHAIN configuration (default)."	},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",			{ 1 },	"Continue testing for up to # minutes."										},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",			{ 1 },	"Continue testing for up to # iterations."									},
		{	GSC_ARG_S32_FLAG,	&args->star,			"-s",			{ 1 },	"The clock and sync signals are in a STAR configuration (default)."			},
		{	GSC_ARG_DEV_INDEX,	&args->index_list[0],	"initiator",	{ 10 },	"The zero based index of the initiator device."								},
		{	GSC_ARG_DEV_INDEX,	&args->index_list[1],	"target 1",		{ 10 },	"The zero based index of target device #1."									},
		{	GSC_ARG_DEV_INDEX,	&args->index_list[2],	"target 2",		{ 10 },	"The zero based index of optional target device #2"							},
		{	GSC_ARG_DEV_INDEX,	&args->index_list[3],	"target 3",		{ 10 },	"The zero based index of optional target device #3"							},
		{	GSC_ARG_DEV_INDEX,	&args->index_list[4],	"target 4",		{ 10 },	"The zero based index of optional target device #4"							},
		{	GSC_ARG_DEV_INDEX,	&args->index_list[5],	"target 5",		{ 10 },	"The zero based index of optional target device #5"							},
		{	GSC_ARG_DEV_INDEX,	&args->index_list[6],	"target 6",		{ 10 },	"The zero based index of optional target device #6"							},
		{	GSC_ARG_DEV_INDEX,	&args->index_list[7],	"target 7",		{ 10 },	"The zero based index of optional target device #7"							},
		{	GSC_ARG_DEV_INDEX,	&args->index_list[8],	"target 8",		{ 10 },	"The zero based index of optional target device #8"							},
		{	GSC_ARG_DEV_INDEX,	&args->index_list[9],	"target 9",		{ 10 },	"The zero based index of optional target device #9"							}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"mbsync",
		/* desc	*/	"Multi-Board Synchronization",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;
	int	i;

	memset(args, 0, sizeof(args_t));

	for (i = 0; i < SIZEOF_ARRAY(args->index_list); i++)
		args->index_list[i]	= -1;

	errs	= gsc_args_parse(argc, argv, &set);

	for (i = 0; i < SIZEOF_ARRAY(args->index_list); i++)
	{
		if (args->index_list[i]	< 0)
			break;
		else
			args->sync_qty	= i + 1;
	}

	if (args->sync_qty == 0)
	{
		errs++;
		gsc_label("Initiator");
		printf("FAIL <---  (Initiator not specified.)\n");
	}
	else if (args->sync_qty == 1)
	{
		errs++;
		gsc_label("Targets");
		printf("FAIL <---  (No targets specified.)\n");
	}

	return(errs);
}



//*****************************************************************************
static int _show_access_index(const args_t* args)
{
	const char*	comma	= "";
	int			errs	= 0;
	int			i;

	gsc_label("Accessing Device");

	for (i = 0; i < args->sync_qty; i++)
	{
		printf("%s%d", comma, args->index_list[i]);
		comma	= ", ";
	}

	printf("\n");
	return(errs);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t	args;
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label_init(28);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		errs	+= _show_access_index(&args);
		errs	+= dsi_init_util(1);
		errs	+= os_id_driver(dsi_open, dsi_read, dsi_close);
		errs	+= dsi_count_boards(1, &args.qty);

		if (args.qty <= 0)
			errs++;

		// Open access to device

		if (errs == 0)
			errs	= service_open(&args);

		if (errs)
			args.continuous	= 0;

		if (errs == 0)
		{
			// Device identification

			// Perform testing

			errs	+= perform_tests(&args, errs);
		}

		// Close device access

		errs	+= service_close(&args);

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((args.qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


