// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/samples/mbsync/main.h $
// $Rev: 53956 $
// $Date: 2023-11-29 06:55:21 -0600 (Wed, 29 Nov 2023) $

// 24DSI: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <ctype.h>
#include <errno.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>

#include "24dsi_main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)			(sizeof((a)) / sizeof((a)[0]))

#define	SAMPLE_RATE				(100000L)
#define	INPUT_FREQUENCY			(5000L)

#define	DATA_SET_SIZE			(250L * 1024)										// -> 256,000
// The data set size is chosen to be just under the overall size of the input
// FIFO.

#define	EXAMINE_OFFSET			(DATA_SET_SIZE / 5)									// -> 51,200
// The first 20% of the buffer is skipped as the system may not have settled.
// This equates to about 50K as follows.
// (250K / 5) = 50K

#define	SAMPLES_NEEDED			((DATA_SET_SIZE * 8/10) / 32)						// -> 5760
// This is the minimum number of channel zero data samples expected in the
// data to be examined. The overall data volume is divided by 32 as the device
// may have 32 channels.

#define	CYCLES_MIN				(SAMPLES_NEEDED  * INPUT_FREQUENCY / SAMPLE_RATE)	// ->288
// This is the minimum number of sine wave cycles expected in the examined
// data. The ratio of (SAMPLES_NEEDED / SAMPLE_RATE) is the fraction of a
// second represented by the volume of data examined. That period of time
// multiplied by the input signal frequency is the number of sine waves
// expected in the examined data.

#define	PERIOD_MIN				(SAMPLES_NEEDED / CYCLES_MIN)
// This is the minimum number of data samples expected for each cycle of the
// input size wave.



// data types *****************************************************************

typedef struct
{
	// Application Settings

	s32	continuous;		// > 0 = ignore errors, < 0 = stop on errors
	s32	index_list[10];	// list of device indexes specified on the command line
	s32	minute_limit;	// for continuous testing
	int	qty;			// Number of boards detected.
	s32	star;
	s32	sync_qty;
	s32	test_limit;		// for continuous testing

	s32	fd_list[10];	// list of devices to access

	// Device Settings

} args_t;

typedef struct
{
	u32		data[DATA_SET_SIZE];
} read_data_t;

typedef struct
{
	int		ignore;
	long	ch_0_cycles;
	float	ch_0_period;
	long	ch_0_samples;
	float	freq;			// frequency of input signal
	s32		fref;			// default
	long	fsamp;			// calculated
	float	vmin;
	float	vmax;
	float	phase_ns;
	float	phase_deg;
} volt_data_t;



// prototypes *****************************************************************

int		multi_board_sync				(const args_t* args, int errs, s32* lp);
int		perform_tests					(const args_t* args, int errs);
int		validate_sync					(const args_t* args, int errs, s32 lp);

int		service_ain_range				(const	args_t* args, int arg);
int		service_autocal					(const	args_t* args);
int		service_clear					(const	args_t* args);
int		service_close					(		args_t* args);
int		service_config					(const	args_t* args, s32 fsamp);
int		service_channel_group_source	(const	args_t* args, int init, int targ);
int		service_channel_order			(const	args_t* args, int arg);
int		service_channel_ready			(const	args_t* args, int init, int targ);
int		service_data_format				(const	args_t* args, int arg);
int		service_data_rate_cycles		(const	args_t* args, volt_data_t* v_data);
int		service_data_rate_examine		(const	args_t* args, const read_data_t* data, volt_data_t* v_data);
int		service_data_rate_frequency		(const	args_t* args, volt_data_t* v_data);
int		service_data_rate_fsamp			(const	args_t* args, volt_data_t* v_data);
int		service_data_rate_period		(const	args_t* args, volt_data_t* v_data);
int		service_data_rate_phase			(const	args_t* args, const read_data_t* data, volt_data_t* v_data, int lp);
int		service_data_width				(const	args_t* args, int arg);
int		service_extern_clock_source		(const	args_t* args, int init, int targ);
int		service_init_mode				(const	args_t* args, int init, int targ);
void	service_list_init				(const	args_t* args);
int		service_low_power				(const	args_t* args, int* lp);
int		service_open					(		args_t* args);
int		service_read					(const	args_t* args, read_data_t* data);
int		service_rx_io_overflow			(const	args_t* args, int init, int targ);
int		service_sw_sync					(const	args_t* args, int init, int targ);
int		service_sw_sync_mode			(const	args_t* args, int init, int targ);
int		service_voltage_range_examine	(const	args_t* args, const read_data_t* data, volt_data_t* v_data, int lp);
int		service_voltage_range_samples	(const	args_t* args, volt_data_t* v_data);
int		service_voltage_range_vmax		(const	args_t* args, volt_data_t* v_data, int lp);
int		service_voltage_range_vmin		(const	args_t* args, volt_data_t* v_data, int lp);



#endif
