// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/samples/rxrate/main.c $
// $Rev: 53964 $
// $Date: 2023-11-29 07:03:13 -0600 (Wed, 29 Nov 2023) $

// 24DSI: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg			values							desc
		{	GSC_ARG_S32_FLAG,	&args->io_mode,			"-bmdma",	{ GSC_IO_MODE_BMDMA },			"Transfer data using Block Mode DMA."					},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",		{ -1 },							"Continue testing until an error occurs."				},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",		{ +1 },							"Continue testing even if errors occur."				},
		{	GSC_ARG_S32_FLAG,	&args->ch_grp_src,		"-cgsa",	{ DSI_CH_GRP_SRC_GEN_A },		"Channel Group Sources: Rate-A Generator (default)."	},
		{	GSC_ARG_S32_FLAG,	&args->ch_grp_src,		"-cgsb",	{ DSI_CH_GRP_SRC_GEN_B },		"Channel Group Sources: Rate-B Generator "				},
		{	GSC_ARG_S32_FLAG,	&args->ch_grp_src,		"-cgsd",	{ DSI_CH_GRP_SRC_DISABLE },		"Channel Group Sources: Disable"						},
		{	GSC_ARG_S32_FLAG,	&args->ch_grp_src,		"-cgsde",	{ DSI_CH_GRP_SRC_DIR_EXTERN },	"Channel Group Sources: Direct External"				},
		{	GSC_ARG_S32_FLAG,	&args->ch_grp_src,		"-cgse",	{ DSI_CH_GRP_SRC_EXTERN },		"Channel Group Sources: External"						},
		{	GSC_ARG_S32_FLAG,	&args->io_mode,			"-dmdma",	{ GSC_IO_MODE_DMDMA },			"Transfer data using Demand Mode DMA."					},
		{	GSC_ARG_S32_RANGE,	&args->fsamp,			"-fsamp",	{ 2000, 200000 },				"Sample data at # samples per second."					},
		{	GSC_ARG_S32_FLAG,	&args->xcvr,			"-lvds",	{ DSI_XCVR_TYPE_LVDS },			"Select LVDS transceivers (default)."					},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",		{ 1 },							"Continue testing for up to # minutes."					},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",		{ 1 },							"Continue testing for up to # iterations."				},
		{	GSC_ARG_S32_RANGE,	&args->ndiv,			"-ndiv",	{ 0, 25 },						"Set Ndiv to # (default is per sample rate)."			},
		{	GSC_ARG_S32_RANGE,	&args->nref,			"-nref",	{ 30, 1000 },					"Set Nref to # (default is per sample rate)."			},
		{	GSC_ARG_S32_RANGE,	&args->nvco,			"-nvco",	{ 30, 1000 },					"Set Nvco to # (default is per sample rate)."			},
		{	GSC_ARG_S32_FLAG,	&args->io_mode,			"-pio",		{ GSC_IO_MODE_PIO },			"Transfer data using PIO (default)."					},
		{	GSC_ARG_S32_MIN,	&args->rx_mb,			"-r",		{ 1 },							"Receive # megabytes (default: 4)."						},
		{	GSC_ARG_S32_RANGE,	&args->seconds,			"-s",		{ 1, 3600 },					"Read data for at least # seconds (default: 15)."		},
		{	GSC_ARG_S32_FLAG,	&args->xcvr,			"-ttl",		{ DSI_XCVR_TYPE_TTL },			"Select TTL transceivers."								},
		{	GSC_ARG_DEV_INDEX,	&args->index,			"index",	{ 1 },							"The zero based index of the device to access."			}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"rxrate",
		/* desc	*/	"Receive Rate Performance Tests",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;

	memset(args, 0, sizeof(args_t));
	args->ch_grp_src	= DSI_CH_GRP_SRC_GEN_A;
	args->fsamp			= 200000;
	args->io_mode		= GSC_IO_MODE_PIO;
	args->ndiv			= -1;
	args->nref			= -1;
	args->nvco			= -1;
	args->rx_mb			= 4;
	args->seconds		= 15;
	args->xcvr			= DSI_XCVR_TYPE_LVDS;

	errs	= gsc_args_parse(argc, argv, &set);

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	gsc_label("Accessing Device");
	printf("%d\n", args->index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t	args;
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label_init(30);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(&args);
		errs	+= dsi_init_util(1);
		errs	+= os_id_driver(dsi_open, dsi_read, dsi_close);
		errs	+= dsi_count_boards(1, &args.qty);

		if (args.qty <= 0)
			errs++;

		// Open access to device

		if (errs == 0)
			errs	= dsi_open_util(args.index, 0, -1, 1, &args.fd);

		if (errs)
			args.continuous	= 0;

		if (errs == 0)
		{
			// Device identification

			errs	= dsi_id_device(args.fd, -1, 1);

			// Perform testing

			errs	+= perform_tests(&args);

			// Close device access

			errs	+= dsi_close_util(args.fd, -1, 1);
		}

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((args.qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


