// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/samples/rxrate/perform.c $
// $Rev: 53964 $
// $Date: 2023-11-29 07:03:13 -0600 (Wed, 29 Nov 2023) $

// 24DSI: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M		(1024 * 1024)	// size of driver's transfer buffer



// variables ******************************************************************

static	u8	_data[_1M];



//*****************************************************************************
static int _configuration(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Configuration");
	errs	+= dsi_config_ai				(args->fd, -1, 0, -1, args->fsamp);
	printf("%s\n", errs ? "FAIL <---" : "PASS");

	errs	+= dsi_rx_io_mode				(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= dsi_rate_gen_a_nvco			(args->fd, -1, 1, args->nvco, NULL);
	errs	+= dsi_rate_gen_b_nvco			(args->fd, -1, 1, args->nvco, NULL);
	errs	+= dsi_rate_gen_a_nref			(args->fd, -1, 1, args->nref, NULL);
	errs	+= dsi_rate_gen_b_nref			(args->fd, -1, 1, args->nref, NULL);
	errs	+= dsi_rate_div_0_ndiv			(args->fd, -1, 1, args->ndiv, NULL);
	errs	+= dsi_rate_div_1_ndiv			(args->fd, -1, 1, args->ndiv, NULL);
	errs	+= dsi_rx_io_overflow			(args->fd, -1, 1, DSI_IO_OVERFLOW_IGNORE, NULL);
	errs	+= dsi_rx_io_underflow			(args->fd, -1, 1, DSI_IO_UNDERFLOW_IGNORE, NULL);
	errs	+= dsi_xcvr_type				(args->fd, -1, 1, args->xcvr, NULL);
	errs	+= dsi_fsamp_ai_report_all		(args->fd, -1, 1, NULL, NULL);
	errs	+= dsi_channel_group_source_all	(args->fd, -1, 1, args->ch_grp_src);
	errs	+= dsi_ain_buf_clear			(args->fd, -1, 1, 0);

	return(errs);
}



//*****************************************************************************
static int _rate_test(const args_t* args, long get)
{
	struct timeval	begin;
	int				errs	= 0;
	long			got;
	long long		limit	= (long long) args->rx_mb * 1000000L;
	struct timeval	minimum;
	struct timeval	now;
	long long		sps;
	long long		total	= 0;
	long			us;

	gsc_label("Reading");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + args->seconds;
	minimum.tv_usec	= begin.tv_usec;

	for (;;)
	{
		got	= dsi_read(args->fd, _data, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", error %ld)\n", got);
			break;
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if (total < limit)
			continue;

		if (now.tv_sec < minimum.tv_sec)
			continue;

		if ((now.tv_sec == minimum.tv_sec) && (now.tv_usec < minimum.tv_usec))
			continue;

		us	= (now.tv_sec - begin.tv_sec) * 1000000L
			+ (now.tv_usec - begin.tv_usec);
		sps	= (total * 10000000L / 4 / us + 5) / 10;

		printf(	"PASS  (");
		gsc_label_long_comma(total / 4);
		printf(" Samples, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
		gsc_label_long_comma(sps);
		printf(" S/S)\n");
		break;
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_data);

	errs	+= _configuration(args);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %ld MB minimum)\n", args->seconds, (long) args->rx_mb);
	gsc_label_level_inc();


	errs	+= _rate_test(args, get);

	errs	+= dsi_ain_buf_overflow		(args->fd, -1, 1, -1, NULL);
	errs	+= dsi_ain_buf_underflow	(args->fd, -1, 1, -1, NULL);

	gsc_label_level_dec();
	return(errs);
}



