// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_ain_buf_clear.c $
// $Rev: 42847 $
// $Date: 2018-05-18 10:29:53 -0500 (Fri, 18 May 2018) $

// 24DSI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi_ain_buf_clear
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI_IOCTL_AIN_BUF_CLEAR service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		at_bndy	Clear the buffer at a scan boundary?
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	dsi_ain_buf_clear(int fd, int index, int verbose, int at_bndy)
{
	int	errs	= 0;
	s32	mode;
	int	ret;

	if (verbose)
		gsc_label_index("ADC Buffer Clear", index);

	if (at_bndy)
	{
		errs	+= dsi_sw_sync_mode(fd, -1, 0, -1, &mode);

		// Clear the buffer with any over or under flow status.
		ret		= dsi_ioctl(fd, DSI_IOCTL_AIN_BUF_CLEAR, NULL);
		errs	+= ret ? 1 : 0;

		// Clear the buffer on a scan boundary.
		errs	+= dsi_sw_sync_mode(fd, -1, 0, DSI_SW_SYNC_MODE_SW_SYNC, NULL);
		errs	+= dsi_sw_sync(fd, -1, 0);
		errs	+= dsi_channels_ready(fd, -1, 0);

		errs	+= dsi_sw_sync_mode(fd, -1, 0, DSI_SW_SYNC_MODE_CLR_BUF, NULL);
		errs	+= dsi_sw_sync(fd, -1, 0);
		errs	+= dsi_channels_ready(fd, -1, 0);

		// Restore the SW Sync Mode.
		errs	+= dsi_sw_sync_mode(fd, -1, 0, mode, NULL);
	}
	else
	{
		ret		= dsi_ioctl(fd, DSI_IOCTL_AIN_BUF_CLEAR, NULL);
		errs	+= ret ? 1 : 0;
	}

	if (verbose)
		printf("%s\n", errs ? "FAIL <---" : "PASS");

	return(errs);
}


