// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_ain_mode.c $
// $Rev: 52688 $
// $Date: 2023-03-24 15:05:09 -0500 (Fri, 24 Mar 2023) $

// 24DSI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi_ain_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI_IOCTL_AIN_MODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	dsi_ain_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	s32		query;
	int		ret;

	if (verbose)
		gsc_label_index("Analog Input Mode", index);

	errs	= dsi_query(fd, -1, 0, DSI_QUERY_D0_1_IN_MODE, &query);

	if (errs)
	{
	}
	else if (query)
	{
		ret		= dsi_ioctl(fd, DSI_IOCTL_AIN_MODE, &set);
		errs	= ret ? 1 : 0;

		switch (set)
		{
			default:

				sprintf(buf, "INVALID: %ld", (long) set);
				break;

			case DSI_AIN_MODE_DIFF:

				strcpy(buf, "Differential");
				break;

			case DSI_AIN_MODE_ZERO:

				strcpy(buf, "Zero Test");
				break;

			case DSI_AIN_MODE_VREF:

				strcpy(buf, "Vref Test");
				break;
		}

		if (verbose)
			printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);
	}
	else
	{
		if (verbose)
			printf("SKIPPED  (Not configurable on this device.)\n");
	}

	if (get)
		get[0]	= set;

	return(errs);
}


