// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_ain_range.c $
// $Rev: 52688 $
// $Date: 2023-03-24 15:05:09 -0500 (Fri, 24 Mar 2023) $

// 24DSI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi_ain_range
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI_IOCTL_AIN_RANGE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	dsi_ain_range(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	int		ret;
	s32		query;

	if (verbose)
		gsc_label_index("Analog Input Range", index);

	errs	= dsi_query(fd, -1, 0, DSI_QUERY_D2_3_RANGE, &query);

	if (errs)
	{
	}
	else if (query)
	{
		ret		= dsi_ioctl(fd, DSI_IOCTL_AIN_RANGE, &set);
		errs	= ret ? 1 : 0;

		switch (set)
		{
			default:

				sprintf(buf, "INVALID: %ld", (long) set);
				break;

			case DSI_AIN_RANGE_2_5V:

				strcpy(buf, "+-2.5v");
				break;

			case DSI_AIN_RANGE_5V:

				strcpy(buf, "+-5v");
				break;

			case DSI_AIN_RANGE_10V:

				strcpy(buf, "+-10v");
				break;
		}

		if (verbose)
			printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);
	}
	else
	{
		if (verbose)
			printf("SKIPPED  (Not configurable on this device.)\n");
	}

	if (get)
		get[0]	= set;

	return(errs);
}


