// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_ch_grp_0_src.c $
// $Rev: 52688 $
// $Date: 2023-03-24 15:05:09 -0500 (Fri, 24 Mar 2023) $

// 24DSI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi_ch_grp_0_src
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI_IOCTL_CH_GRP_0_SRC service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	dsi_ch_grp_0_src(int fd, int index, int verbose, s32 set, s32* get)
{
	#define	GROUP	0

	char	buf[128];
	int		errs	= 0;
	s32		groups;
	s32		pri;
	int		ret;

	if (verbose)
		gsc_label_index("Channel Group 0 Source", index);

	errs	+= dsi_query(fd, -1, 0, DSI_QUERY_CHANNEL_GPS, &groups);
	errs	+= dsi_query(fd, -1, 0, DSI_QUERY_CH_GRP_0_RAR, &pri);

	if (errs)
	{
	}
	else if (groups < 1)
	{
		if (verbose)
		{
			printf(	"SKIPPED  "
					"(channel group %ld not supported on this device)\n",
					(long) GROUP);
		}
	}
	else
	{
		if ((groups) && (pri))
		{
			set		= (set == DSI_CH_GRP_SRC_DISABLE)
					? DSI_CH_GRP_SRC_DISABLE
					: DSI_CH_GRP_SRC_ENABLE;
		}

		ret		= dsi_ioctl(fd, DSI_IOCTL_CH_GRP_0_SRC, &set);
		errs	= ret ? 1 : 0;

		switch (set)
		{
			default:

				sprintf(buf, "INVALID: %ld", (long) set);
				break;

			case DSI_CH_GRP_SRC_GEN_A:

				strcpy(buf, "Rate Generator A");
				break;

			case DSI_CH_GRP_SRC_GEN_B:

				strcpy(buf, "Rate Generator B");
				break;
			case DSI_CH_GRP_SRC_EXTERN:

				strcpy(buf, "External");
				break;

			case DSI_CH_GRP_SRC_DIR_EXTERN:

				strcpy(buf, "Direct External");
				break;

			case DSI_CH_GRP_SRC_DISABLE:

				strcpy(buf, "Disable");
				break;

			case DSI_CH_GRP_SRC_ENABLE:

				strcpy(buf, "Enable");
				break;
		}

		if (verbose)
			printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


