// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_config_ai.c $
// $Rev: 53949 $
// $Date: 2023-11-29 06:46:53 -0600 (Wed, 29 Nov 2023) $

// 24DSI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi_config_ai
*
*	Purpose:
*
*		Configure the given device using common defaults.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely? If not, then work quietly.
*
*		fref	This is the PLL Fref value, or -1 to use the default.
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	dsi_config_ai(int fd, int index, int verbose, s32 fref, s32 fsamp)
{
	int	errs	= 0;
	s32	ndiv;
	s32	nrate;	// Legacy
	s32	nref;	// PLL
	s32	nvco;	// PLL

	if (verbose)
	{
		gsc_label_index("A/I Configuration", index);
		printf("\n");
		gsc_label_level_inc();
	}

	errs	+= dsi_initialize				(fd, index, verbose);

	// Settings must be applied after the initialization call!

	errs	+= dsi_rx_io_mode				(fd, index, verbose, GSC_IO_MODE_PIO,				NULL);
	errs	+= dsi_rx_io_overflow			(fd, index, verbose, DSI_IO_OVERFLOW_CHECK,			NULL);
	errs	+= dsi_rx_io_timeout			(fd, index, verbose, 30,							NULL);
	errs	+= dsi_rx_io_underflow			(fd, index, verbose, DSI_IO_UNDERFLOW_CHECK,		NULL);
	errs	+= dsi_ain_mode					(fd, index, verbose, DSI_AIN_MODE_DIFF,				NULL);
	errs	+= dsi_ain_range				(fd, index, verbose, DSI_AIN_RANGE_5V,				NULL);
	errs	+= dsi_ain_buf_input			(fd, index, verbose, DSI_AIN_BUF_INPUT_ENABLE,		NULL);
	errs	+= dsi_ain_buf_threshold		(fd, index, verbose, 128L * 1024,					NULL);
	errs	+= dsi_sw_sync_mode				(fd, index, verbose, DSI_SW_SYNC_MODE_CLR_BUF,		NULL);
	errs	+= dsi_channel_order			(fd, index, verbose, DSI_CHANNEL_ORDER_SYNC,		NULL);
	errs	+= dsi_data_format				(fd, index, verbose, DSI_DATA_FORMAT_OFF_BIN,		NULL);
	errs	+= dsi_data_width				(fd, index, verbose, DSI_DATA_WIDTH_16,				NULL);
	errs	+= dsi_init_mode				(fd, index, verbose, DSI_INIT_MODE_INITIATOR,		NULL);
	errs	+= dsi_ext_clk_src				(fd, index, verbose, DSI_EXT_CLK_SRC_GRP_0,			NULL);
	errs	+= dsi_fref_compute				(fd, index, verbose, &fref);
	errs	+= dsi_fsamp_ai_compute			(fd, index, verbose, fref, fsamp, &nvco, &nref, &nrate, &ndiv, NULL);
	errs	+= dsi_rate_gen_x_nvco_all		(fd, index, verbose, nvco);
	errs	+= dsi_rate_gen_x_nref_all		(fd, index, verbose, nref);
	errs	+= dsi_rate_gen_x_nrate_all		(fd, index, verbose, nrate);
	errs	+= dsi_rate_div_x_ndiv_all		(fd, index, verbose, ndiv);
	errs	+= dsi_fsamp_ai_report_all		(fd, index, verbose, &fref,							NULL);
	errs	+= dsi_xcvr_type				(fd, index, verbose, DSI_XCVR_TYPE_LVDS,			NULL);

	errs	+= dsi_channel_group_source_all	(fd, index, verbose, DSI_CH_GRP_SRC_GEN_A);

	// Settings affecting the sample rate or voltage range must generally
	// be applied before autocalibration.
	// Refer to the board user manual for clarification.
	errs	+= dsi_autocal					(fd, index, verbose);

	// Interrupt selections should be applied after autocalibration as the
	// driver overwrites the current selection to detect the Autocalibration
	// Done interrupt. If an application intends to detect the Autocalibration
	// Done interrupt, then the interrupt selection may be made beforehand.
	errs	+= dsi_irq_sel					(fd, index, verbose, DSI_IRQ_INIT_DONE,				NULL);

	// The final step is to clear the buffer. This service clears the FIFO and
	// resets the overflow and underflow status bits.
	errs	+= dsi_ain_buf_clear			(fd, index, verbose, 1);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


