// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_ext_clk_src.c $
// $Rev: 53949 $
// $Date: 2023-11-29 06:46:53 -0600 (Wed, 29 Nov 2023) $

// 24DSI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi_ext_clk_src
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI_IOCTL_EXT_CLK_SRC service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	dsi_ext_clk_src(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	int		ret;

	if (verbose)
		gsc_label_index("Ext. Clock Source", index);

	ret		= dsi_ioctl(fd, DSI_IOCTL_EXT_CLK_SRC, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case DSI_EXT_CLK_SRC_GRP_0:

			strcpy(buf, "Channel Group 0");
			break;

		case DSI_EXT_CLK_SRC_GEN_A:

			strcpy(buf, "Rate Generator A");
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);

	if (get)
		get[0]	= set;

	return(errs);
}


