// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_irq_sel.c $
// $Rev: 53949 $
// $Date: 2023-11-29 06:46:53 -0600 (Wed, 29 Nov 2023) $

// 24DSI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi_irq_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI_IOCTL_IRQ_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	dsi_irq_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	int		ret;

	if (verbose)
		gsc_label_index("IRQ Select", index);

	ret		= dsi_ioctl(fd, DSI_IOCTL_IRQ_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case DSI_IRQ_INIT_DONE:

			strcpy(buf, "Initialize Done");
			break;

		case DSI_IRQ_AUTOCAL_DONE:

			strcpy(buf, "Autocalibration Done");
			break;

		case DSI_IRQ_CHAN_READY:

			strcpy(buf, "Channel Ready");
			break;

		case DSI_IRQ_AIN_BUF_THRESH_L2H:

			strcpy(buf, "Buffer Threshold Low-to-High");
			break;

		case DSI_IRQ_AIN_BUF_THRESH_H2L:

			strcpy(buf, "Buffer Threshold High-to-Low");
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);

	if (get)
		get[0]	= set;

	return(errs);
}


