// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_query.c $
// $Rev: 53949 $
// $Date: 2023-11-29 06:46:53 -0600 (Wed, 29 Nov 2023) $

// 24DSI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi_query
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case DSI_QUERY_AUTOCAL_MS:			ptr	= "Autocal Period";				break;
		case DSI_QUERY_CH_GRP_0_RAR:		ptr = "RAR @ Chan Grp 0";			break;
		case DSI_QUERY_CHANNEL_GPS:			ptr = "Channel Groups";				break;
		case DSI_QUERY_CHANNEL_MAX:			ptr	= "Max Channels";				break;
		case DSI_QUERY_CHANNEL_QTY:			ptr	= "Input Channels";				break;
		case DSI_QUERY_COUNT:				ptr	= "Query Options";				break;
		case DSI_QUERY_DEVICE_TYPE:			ptr	= "Device Type";				break;
		case DSI_QUERY_D0_1_IN_MODE:		ptr = "BCTLR.D0/1: A/I Mode";		break;
		case DSI_QUERY_D15_PLL:				ptr = "BCFGR.D15: PLL";				break;
		case DSI_QUERY_D16_CHANNELS:		ptr = "BCFGR.D16: Channel count";	break;
		case DSI_QUERY_D16_CHANNELS_0:		ptr = "BCFGR.D16: Channel count";	break;
		case DSI_QUERY_D16_SYNC_1:			ptr = "BCTLR.D16: 1 for Synch";		break;
		case DSI_QUERY_D17_18_RANGE:		ptr = "BCFGR.D17/18: Volt Range";	break;
		case DSI_QUERY_D17_CHANNELS:		ptr = "BCFGR.D17: Channel Count";	break;
		case DSI_QUERY_D17_CHANNELS_0:		ptr = "BCFGR.D17: Channel count";	break;
		case DSI_QUERY_D18_CF_FILT:			ptr = "BCFGR.D18: Custom Filt";		break;
		case DSI_QUERY_D18_RIO:				ptr = "BCFGR.D18: Rear I/O";		break;
		case DSI_QUERY_D19_20_FILT:			ptr = "BCFGR.D19/20: Filter Freq";	break;
		case DSI_QUERY_D19_EXT_TEMP:		ptr = "BCFGR.D19: Extended Temp";	break;
		case DSI_QUERY_D19_FREQ_FILT:		ptr = "BCTLR.D19: Freq Filter";		break;
		case DSI_QUERY_D2_3_RANGE:			ptr = "BCTLR.D2/3: Volt Range";		break;
		case DSI_QUERY_D20_LOW_POWER:		ptr = "BCFGR.D20: Low Power";		break;
		case DSI_QUERY_D20_XCVR:			ptr = "BCTLR.D20: Xcvr";			break;
		case DSI_QUERY_D21_EXT_TEMP:		ptr = "BCFGR.D21: Extended Temp";	break;
		case DSI_QUERY_D21_EXT_TRIG:		ptr = "BCTLR.D21: Arm Ext Trigg";	break;
		case DSI_QUERY_D22_24DSI6LN:		ptr = "BCFGR.D22: 24DSI6LN";		break;
		case DSI_QUERY_D22_COUPLING:		ptr = "BCTLR.D22: Coupling";		break;
		case DSI_QUERY_D22_THR_FLAG:		ptr = "BCTLR.D22: Thresh Flag Out";	break;
		case DSI_QUERY_D23_INV_EXT_TRIG:	ptr = "BCTLR.D23: Ext Trig Pol";	break;
		case DSI_QUERY_D23_RATE_OUT:		ptr = "BCTLR.D23: ADC Rate Out";	break;
		case DSI_QUERY_FGEN_MAX:			ptr	= "Fgen Max";					break;
		case DSI_QUERY_FGEN_MIN:			ptr	= "Fgen Min";					break;
		case DSI_QUERY_FIFO_SIZE:			ptr	= "FIFO Size";					break;
		case DSI_QUERY_FREF_DEFAULT:		ptr	= "Fref Default";				break;
		case DSI_QUERY_FSAMP_DEFAULT:		ptr = "Fsamp Default";				break;
		case DSI_QUERY_FSAMP_MAX:			ptr	= "Fsamp Max";					break;
		case DSI_QUERY_FSAMP_MIN:			ptr	= "Fsamp Min";					break;
		case DSI_QUERY_GPS_PRESENT:			ptr = "GPS Board";					break;
		case DSI_QUERY_INIT_MS:				ptr	= "Initialize Period";			break;
		case DSI_QUERY_LOW_NOISE:			ptr = "Low Noise Board";			break;
		case DSI_QUERY_LOW_POWER:			ptr = "Low Power Board";			break;
		case DSI_QUERY_NDIV_MASK:			ptr = "Ndiv Mask";					break;
		case DSI_QUERY_NDIV_MAX:			ptr	= "Ndiv Max";					break;
		case DSI_QUERY_NDIV_MIN:			ptr	= "Ndiv Min";					break;
		case DSI_QUERY_NRATE_MASK:			ptr = "Nrate Mask";					break;
		case DSI_QUERY_NRATE_MAX:			ptr = "Nrate Max";					break;
		case DSI_QUERY_NRATE_MIN:			ptr = "Nrate Min";					break;
		case DSI_QUERY_NREF_MASK:			ptr = "Nref Mask";					break;
		case DSI_QUERY_NREF_MAX:			ptr	= "Nref Max";					break;
		case DSI_QUERY_NREF_MIN:			ptr	= "Nref Min";					break;
		case DSI_QUERY_NVCO_MASK:			ptr = "Nvco Mask";					break;
		case DSI_QUERY_NVCO_MAX:			ptr	= "Nvco Max";					break;
		case DSI_QUERY_NVCO_MIN:			ptr	= "Nvco Min";					break;
		case DSI_QUERY_PLL_PRESENT:			ptr = "Clocking Method";			break;
		case DSI_QUERY_RATE_DIV_QTY:		ptr = "Rate Dividers";				break;
		case DSI_QUERY_RATE_GEN_QTY:		ptr = "Rate Generators";			break;
		case DSI_QUERY_RCAR_RCBR:			ptr = "Reg: RCAR & RCBR";			break;
		case DSI_QUERY_REG_ICMR:			ptr = "Reg: ICMR";					break;
		case DSI_QUERY_RFCR:				ptr = "Reg: RFCR";					break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= dsi_ioctl(fd, DSI_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case DSI_QUERY_CHANNEL_GPS:
		case DSI_QUERY_CHANNEL_MAX:
		case DSI_QUERY_CHANNEL_QTY:
		case DSI_QUERY_COUNT:
		case DSI_QUERY_RATE_DIV_QTY:
		case DSI_QUERY_RATE_GEN_QTY:
		case DSI_QUERY_NDIV_MAX:
		case DSI_QUERY_NDIV_MIN:
		case DSI_QUERY_NRATE_MAX:
		case DSI_QUERY_NRATE_MIN:
		case DSI_QUERY_NREF_MAX:
		case DSI_QUERY_NREF_MIN:
		case DSI_QUERY_NVCO_MAX:
		case DSI_QUERY_NVCO_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case DSI_QUERY_NDIV_MASK:
		case DSI_QUERY_NRATE_MASK:
		case DSI_QUERY_NREF_MASK:
		case DSI_QUERY_NVCO_MASK:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case DSI_QUERY_FGEN_MAX:
		case DSI_QUERY_FGEN_MIN:
		case DSI_QUERY_FREF_DEFAULT:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case DSI_QUERY_AUTOCAL_MS:
		case DSI_QUERY_INIT_MS:

			if (set == 0)
			{
				strcpy(buf, "Feature Not Supported");
			}
			else
			{
				gsc_label_long_comma_buf(set, buf);
				strcat(buf, " ms");
			}

			break;

		case DSI_QUERY_FSAMP_DEFAULT:
		case DSI_QUERY_FSAMP_MAX:
		case DSI_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case DSI_QUERY_CH_GRP_0_RAR:
		case DSI_QUERY_GPS_PRESENT:
		case DSI_QUERY_LOW_POWER:
		case DSI_QUERY_RCAR_RCBR:
		case DSI_QUERY_RFCR:
		case DSI_QUERY_D0_1_IN_MODE:
		case DSI_QUERY_D2_3_RANGE:
		case DSI_QUERY_D19_FREQ_FILT:
		case DSI_QUERY_D22_COUPLING:
		case DSI_QUERY_D23_RATE_OUT:
		case DSI_QUERY_D17_18_RANGE:
		case DSI_QUERY_D19_20_FILT:
		case DSI_QUERY_D21_EXT_TEMP:
		case DSI_QUERY_D22_24DSI6LN:
		case DSI_QUERY_LOW_NOISE:
		case DSI_QUERY_D23_INV_EXT_TRIG:
		case DSI_QUERY_REG_ICMR:
		case DSI_QUERY_D15_PLL:
		case DSI_QUERY_D18_CF_FILT:
		case DSI_QUERY_D18_RIO:
		case DSI_QUERY_D19_EXT_TEMP:
		case DSI_QUERY_D20_LOW_POWER:
		case DSI_QUERY_D20_XCVR:
		case DSI_QUERY_D21_EXT_TRIG:
		case DSI_QUERY_D22_THR_FLAG:
		case DSI_QUERY_D16_SYNC_1:
		case DSI_QUERY_PLL_PRESENT:

			sprintf(buf, "%s", set ? "Supported" : "Not Supported");
			break;

		case DSI_QUERY_D16_CHANNELS_0:
		case DSI_QUERY_D17_CHANNELS_0:

			if (set)
				sprintf(buf, "0 = %ld Channels", (long) set);
			else
				strcpy(buf, "No Channels Specified");

			break;

		case DSI_QUERY_D16_CHANNELS:
		case DSI_QUERY_D17_CHANNELS:

			if (set)
				sprintf(buf, "1 = %ld Channels", (long) set);
			else
				strcpy(buf, "Not Related to Channel Count");

			break;

		case DSI_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case DSI_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_24DSI6)
			{
				strcpy(buf, "24DSI6");
			}
			else if (set == GSC_DEV_TYPE_24DSI12)
			{
				strcpy(buf, "24DSI12");
			}
			else if (set == GSC_DEV_TYPE_24DSI32)
			{
				strcpy(buf, "24DSI32");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


