// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_rate_div_x_ndiv.c $
// $Rev: 52688 $
// $Date: 2023-03-24 15:05:09 -0500 (Fri, 24 Mar 2023) $

// 24DSI: Utilities: source file

#include "main.h"



// variables ******************************************************************

static const int	_ndiv_ioctl[]	=
{
	DSI_IOCTL_RATE_DIV_0_NDIV,
	DSI_IOCTL_RATE_DIV_1_NDIV
};



/******************************************************************************
*
*	Function:	dsi_rate_div_x_ndiv
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI_IOCTL_RATE_DIV_X_NDIV services.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		divider	The index of the rate divider to access.
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi_rate_div_x_ndiv(int fd, int index, int verbose, int divider, s32 set, s32* get)
{
	char	buf[128];
	int		cmd;
	int		errs	= 0;
	s32		groups;
	s32		max;
	s32		min;

	if (verbose)
	{
		sprintf(buf, "Rate Divider %d Ndiv", divider);
		gsc_label_index(buf, index);
	}

	for (;;)	// A convenience loop.
	{
		if ((divider < 0) || (divider > 1))
		{
			errs	= 1;

			if (verbose)
				printf("FAIL <---  (invalid rate divider index: %d)\n", divider);

			break;
		}

		cmd		= _ndiv_ioctl[divider];
		errs	+= dsi_query(fd, -1, 0, DSI_QUERY_CHANNEL_GPS, &groups);
		errs	+= dsi_query(fd, -1, 0, DSI_QUERY_NDIV_MAX, &max);
		errs	+= dsi_query(fd, -1, 0, DSI_QUERY_NDIV_MIN, &min);

		if (errs)
		{
			break;
		}
		else if (divider >= groups)
		{
			if (verbose)
				printf("SKIPPED  (Not supported on this device.)\n");

			break;
		}
		else
		{
			set		= (set == -1) ? -1 : (set < min) ? min : ((set > max) ? max : set);
			errs	= dsi_ioctl(fd, cmd, &set);

			if (verbose)
				printf("%s  (Ndiv = %ld)\n", errs ? "FAIL <---" : "PASS", (long) set);
		}

		break;
	}

	if (get)
		get[0]	= set;

	return(errs);
}



/******************************************************************************
*
*	Function:	dsi_rate_div_x_ndiv_all
*
*	Purpose:
*
*		Provide a visual wrapper for updating all of the rate dividers.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi_rate_div_x_ndiv_all(int fd, int index, int verbose, s32 set)
{
	int	errs;
	int	divider;
	s32	qty;

	if (verbose)
			gsc_label_index("Ndiv", index);

	errs	= dsi_query(fd, -1, 0, DSI_QUERY_RATE_DIV_QTY, &qty);

	if (errs)
	{
	}
	else
	{
		if (verbose)
			printf("\n");

		gsc_label_level_inc();

		for (divider = 0; divider < qty; divider++)
			errs	+= dsi_rate_div_x_ndiv(fd, index, verbose, divider, set, NULL);

		gsc_label_level_dec();
	}

	return(errs);
}


