// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_rate_gen_b_nvco.c $
// $Rev: 52688 $
// $Date: 2023-03-24 15:05:09 -0500 (Fri, 24 Mar 2023) $

// 24DSI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi_rate_gen_b_nvco
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI_IOCTL_RATE_GEN_B_NVCO service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi_rate_gen_b_nvco(int fd, int index, int verbose, s32 set, s32* get)
{
	int	errs	= 0;
	s32	gen_qty;
	s32	max;
	s32	min;
	s32	pll;
	int	ret;

	if (verbose)
		gsc_label_index("Rate Gen B Nvco", index);

	errs	+= dsi_query(fd, -1, 0, DSI_QUERY_PLL_PRESENT, &pll);
	errs	+= dsi_query(fd, -1, 0, DSI_QUERY_RATE_GEN_QTY, &gen_qty);
	errs	+= dsi_query(fd, -1, 0, DSI_QUERY_NVCO_MAX, &max);
	errs	+= dsi_query(fd, -1, 0, DSI_QUERY_NVCO_MIN, &min);

	if (errs)
	{
	}
	else if ((gen_qty < 2) || (pll == 0))
	{
		if (verbose)
			printf("SKIPPED  (Not supported on this device.)\n");
	}
	else
	{
		set		= (set == -1) ? -1 : (set < min) ? min : ((set > max) ? max : set);
		ret		= dsi_ioctl(fd, DSI_IOCTL_RATE_GEN_B_NVCO, &set);
		errs	= ret ? 1 : 0;

		if (verbose)
			printf("%s  (Nvco = %ld)\n", errs ? "FAIL <---" : "PASS", (long) set);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


