// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_rate_gen_x_nrate.c $
// $Rev: 52016 $
// $Date: 2022-11-21 09:01:02 -0600 (Mon, 21 Nov 2022) $

// 24DSI: Utilities: source file

#include "main.h"



// variables ******************************************************************

static const int	_nrate_ioctl[]	=
{
	DSI_IOCTL_RATE_GEN_A_NRATE,
	DSI_IOCTL_RATE_GEN_B_NRATE
};



//*****************************************************************************
int dsi_rate_gen_x_nrate(int fd, int index, int verbose, int gen, s32 set, s32* get)
{
	char	buf[128];
	int		cmd;
	int		errs	= 0;
	s32		gen_qty;
	s32		max;
	s32		min;
	s32		pll;

	if (verbose)
	{
		sprintf(buf, "Rate Gen %c Nrate", 'A' + gen);
		gsc_label_index(buf, index);
	}

	for (;;)	// A convenience loop.
	{
		if ((gen < 0) || (gen > 1))
		{
			errs	= 1;

			if (verbose)
				printf("FAIL <---  (invalid rate generator index: %d)\n", gen);

			break;
		}

		cmd		= _nrate_ioctl[gen];
		errs	+= dsi_query(fd, -1, 0, DSI_QUERY_PLL_PRESENT, &pll);
		errs	+= dsi_query(fd, -1, 0, DSI_QUERY_RATE_GEN_QTY, &gen_qty);
		errs	+= dsi_query(fd, -1, 0, DSI_QUERY_NRATE_MAX, &max);
		errs	+= dsi_query(fd, -1, 0, DSI_QUERY_NRATE_MIN, &min);

		if (errs)
		{
		}
		else if (gen >= gen_qty)
		{
			if (verbose)
				printf("SKIPPED  (Not supported on this device.)\n");
		}
		else if (pll)
		{
			if (verbose)
				printf("SKIPPED  (Not supported on this device.)\n");
		}
		else
		{
			set		= (set == -1) ? -1 : (set < min) ? min : ((set > max) ? max : set);
			errs	= dsi_ioctl(fd, cmd, &set);

			if (verbose)
				printf("%s  (Nrate = %ld)\n", errs ? "FAIL <---" : "PASS", (long) set);
		}

		break;
	}

	if (get)
		get[0]	= set;

	return(errs);
}


//*****************************************************************************
int dsi_rate_gen_x_nrate_all(int fd, int index, int verbose, s32 set)
{
	int	errs	= 0;
	int	gen;
	s32	gen_qty;
	s32	pll;

	if (verbose)
			gsc_label_index("Nrate", index);

	errs	+= dsi_query(fd, -1, 0, DSI_QUERY_PLL_PRESENT, &pll);
	errs	+= dsi_query(fd, -1, 0, DSI_QUERY_RATE_GEN_QTY, &gen_qty);

	if (errs)
	{
	}
	else if (pll)
	{
		if (verbose)
			printf("SKIPPED  (Not supported on this device.)\n");
	}
	else
	{
		if (verbose)
			printf("\n");

		gsc_label_level_inc();

		for (gen = 0; gen < gen_qty; gen++)
			errs	+= dsi_rate_gen_x_nrate(fd, index, verbose, gen, set, NULL);

		gsc_label_level_dec();
	}

	return(errs);
}


