// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI/24DSI_Linux_3.x_and_4.x_GSC_DN/utils/util_ta_inv_ext_trigger.c $
// $Rev: 52688 $
// $Date: 2023-03-24 15:05:09 -0500 (Fri, 24 Mar 2023) $

// 24DSI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi_ta_inv_ext_trigger
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI_IOCTL_TA_INV_EXT_TRIGGER service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	dsi_ta_inv_ext_trigger(int fd, int index, int verbose, s32 set, s32* get)
{
	char	buf[128];
	int		errs;
	int		ret;

	if (verbose)
		gsc_label_index("TA Invert External Trigger", index);

	ret		= dsi_ioctl(fd, DSI_IOCTL_TA_INV_EXT_TRIGGER, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			sprintf(buf, "INVALID: %ld", (long) set);
			break;

		case DSI_TA_INV_EXT_TRIGGER_NO:

			strcpy(buf, "No (Rising Edge)");
			break;

		case DSI_TA_INV_EXT_TRIGGER_YES:

			strcpy(buf, "Yes (Falling Edge)");
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);

	if (get)
		get[0]	= set;

	return(errs);
}


